@@ -1,4 +1,4 @@
-let p = Unix.getpid() in
+let _ = Unix.getpid() in
 
 print_string "OK\n";;
 
