@@ -1,4 +1,4 @@
-(* $Id: findlib.ml 81 2005-11-27 20:10:34Z gerd $
+(* $Id: findlib.ml 189 2013-01-14 17:10:28Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -12,6 +12,7 @@ exception Package_loop
 
 
 
+let init_called = ref false ;;
 
 let conf_default_location = ref "";;
 let conf_meta_directory = ref "";;
@@ -19,10 +20,13 @@ let conf_search_path = ref [];;
 let conf_command = ref [];;
 let conf_stdlib = ref "";;
 let conf_ldconf = ref "";;
+let conf_ignore_dups_in = ref (None : string option);;
 
 let ocamlc_default = "ocamlc";;
 let ocamlopt_default = "ocamlopt";;
 let ocamlcp_default = "ocamlcp";;
+let ocamloptp_default = "ocamloptp";;
+let ocamlmklib_default = "ocamlmklib";;
 let ocamlmktop_default = "ocamlmktop";;
 let ocamldep_default = "ocamldep";;
 let ocamlbrowser_default = "ocamlbrowser";;
@@ -33,10 +37,13 @@ let init_manually
       ?(ocamlc_command = ocamlc_default)
       ?(ocamlopt_command = ocamlopt_default)
       ?(ocamlcp_command = ocamlcp_default)
+      ?(ocamloptp_command = ocamloptp_default)
+      ?(ocamlmklib_command = ocamlmklib_default)
       ?(ocamlmktop_command = ocamlmktop_default)
       ?(ocamldep_command = ocamldep_default)
       ?(ocamlbrowser_command = ocamlbrowser_default)
       ?(ocamldoc_command = ocamldoc_default)
+      ?ignore_dups_in
       ?(stdlib = Findlib_config.ocaml_stdlib)
       ?(ldconf = Findlib_config.ocaml_ldconf)
       ~install_dir
@@ -45,6 +52,8 @@ let init_manually
   conf_command := [ `ocamlc,     ocamlc_command;
 		    `ocamlopt,   ocamlopt_command;
 		    `ocamlcp,    ocamlcp_command;
+		    `ocamloptp,  ocamloptp_command;
+		    `ocamlmklib, ocamlmklib_command;
 		    `ocamlmktop, ocamlmktop_command;
 		    `ocamldep,   ocamldep_command;
 		    `ocamlbrowser, ocamlbrowser_command;
@@ -55,7 +64,9 @@ let init_manually
   conf_meta_directory := meta_dir;
   conf_stdlib := stdlib;
   conf_ldconf := ldconf;
-  Fl_package_base.init !conf_search_path stdlib
+  conf_ignore_dups_in := ignore_dups_in;
+  Fl_package_base.init !conf_search_path stdlib !conf_ignore_dups_in;
+  init_called := true
 ;;
 
 
@@ -84,7 +95,8 @@ let command_names cmd_spec =
 
 let init
       ?env_ocamlpath ?env_ocamlfind_destdir ?env_ocamlfind_metadir
-      ?env_ocamlfind_commands ?env_camllib ?env_ldconf
+      ?env_ocamlfind_commands ?env_ocamlfind_ignore_dups_in
+      ?env_camllib ?env_ldconf
       ?config ?toolchain () =
   
   let config_file =
@@ -127,11 +139,17 @@ let init
       | None -> []
       | Some p -> [p] in
 
-  let sys_ocamlc, sys_ocamlopt, sys_ocamlcp, sys_ocamlmktop, sys_ocamldep,
-      sys_ocamlbrowser, sys_ocamldoc,
+  let sys_ocamlc, sys_ocamlopt, sys_ocamlcp, sys_ocamloptp, sys_ocamlmklib,
+      sys_ocamlmktop, sys_ocamldep, sys_ocamlbrowser, sys_ocamldoc,
       sys_search_path, sys_destdir, sys_metadir, sys_stdlib, sys_ldconf = 
     (
       let config_vars =
+        if config_file <> "" &&
+           not(Sys.file_exists config_file) && not(Sys.file_exists configd_file)
+        then
+          failwith("Config file not found - neither " ^ 
+                     config_file ^ " nor the directory " ^ 
+                     configd_file);
 	if Sys.file_exists config_file then 
 	  vars_of_file config_file
 	else
@@ -143,27 +161,43 @@ let init
 	  [] in
       let vars = config_vars @ configd_vars in
       if vars <> [] then (
+        let found = ref false in
 	let lookup name default =
-	  try Fl_metascanner.lookup name config_preds vars
+          let explicit_preds =
+            List.for_all
+              (fun p -> Fl_metascanner.predicate_exists p vars) 
+              config_preds in
+          found := !found || explicit_preds;
+	  try 
+            Fl_metascanner.lookup name config_preds vars
 	  with Not_found -> default
 	in
-	( (lookup "ocamlc" ocamlc_default),
-	  (lookup "ocamlopt" ocamlopt_default),
-	  (lookup "ocamlcp" ocamlcp_default),
-	  (lookup "ocamlmktop" ocamlmktop_default),
-	  (lookup "ocamldep" ocamldep_default),
-	  (lookup "ocamlbrowser" ocamlbrowser_default),
-	  (lookup "ocamldoc" ocamldoc_default),
-	  Fl_split.path (lookup "path" ""),
-	  (lookup "destdir" ""),
-	  (lookup "metadir" "none"),
-	  (lookup "stdlib" Findlib_config.ocaml_stdlib),
-	  (lookup "ldconf" Findlib_config.ocaml_ldconf)
-	)
+        let config_tuple =
+	  ( (lookup "ocamlc" ocamlc_default),
+	    (lookup "ocamlopt" ocamlopt_default),
+	    (lookup "ocamlcp" ocamlcp_default),
+	    (lookup "ocamloptp" ocamloptp_default),
+	    (lookup "ocamlmklib" ocamlmklib_default),
+	    (lookup "ocamlmktop" ocamlmktop_default),
+	    (lookup "ocamldep" ocamldep_default),
+	    (lookup "ocamlbrowser" ocamlbrowser_default),
+	    (lookup "ocamldoc" ocamldoc_default),
+	    Fl_split.path (lookup "path" ""),
+	    (lookup "destdir" ""),
+	    (lookup "metadir" "none"),
+	    (lookup "stdlib" Findlib_config.ocaml_stdlib),
+	    (lookup "ldconf" Findlib_config.ocaml_ldconf)
+	  ) in
+        if not !found && config_preds <> [] then
+          prerr_endline("ocamlfind: [WARNING] Undefined toolchain: " ^ 
+                          String.concat "" config_preds);
+        config_tuple
       )
       else
-	( ocamlc_default, ocamlopt_default, ocamlcp_default, ocamlmktop_default,
-	  ocamldep_default, ocamlbrowser_default, ocamldoc_default,
+	( ocamlc_default, ocamlopt_default, ocamlcp_default, ocamloptp_default,
+	  ocamlmklib_default,
+	  ocamlmktop_default, ocamldep_default, ocamlbrowser_default,
+	  ocamldoc_default,
 	  [],
 	  "",
           "none",
@@ -215,13 +249,21 @@ let init
       | None ->
 	  try Sys.getenv "OCAMLFIND_LDCONF" with Not_found -> ""
   in
+  let ignore_dups_in =
+    match  env_ocamlfind_ignore_dups_in with
+      | Some x -> Some x
+      | None ->
+	  try Some(Sys.getenv "OCAMLFIND_IGNORE_DUPS_IN") 
+	  with Not_found -> None in
 
-  let ocamlc, ocamlopt, ocamlcp, ocamlmktop, ocamldep, ocamlbrowser,
-      ocamldoc,
+  let ocamlc, ocamlopt, ocamlcp, ocamloptp, ocamlmklib, ocamlmktop,
+      ocamldep, ocamlbrowser, ocamldoc,
       search_path, destdir, metadir, stdlib, ldconf =
     (try List.assoc "ocamlc"     env_commands with Not_found -> sys_ocamlc),
     (try List.assoc "ocamlopt"   env_commands with Not_found -> sys_ocamlopt),
     (try List.assoc "ocamlcp"    env_commands with Not_found -> sys_ocamlcp),
+    (try List.assoc "ocamloptp"  env_commands with Not_found -> sys_ocamloptp),
+    (try List.assoc "ocamlmklib" env_commands with Not_found -> sys_ocamlmklib),
     (try List.assoc "ocamlmktop" env_commands with Not_found -> sys_ocamlmktop),
     (try List.assoc "ocamldep"   env_commands with Not_found -> sys_ocamldep),
     (try List.assoc "ocamlbrowser" env_commands with Not_found -> sys_ocamlbrowser),
@@ -237,10 +279,13 @@ let init
     ~ocamlc_command: ocamlc
     ~ocamlopt_command: ocamlopt
     ~ocamlcp_command: ocamlcp
+    ~ocamloptp_command: ocamloptp
+    ~ocamlmklib_command: ocamlmklib
     ~ocamlmktop_command: ocamlmktop
     ~ocamldep_command: ocamldep
     ~ocamlbrowser_command: ocamlbrowser
     ~ocamldoc_command: ocamldoc
+    ?ignore_dups_in
     ~stdlib: stdlib
     ~ldconf: ldconf
     ~install_dir: destdir
@@ -250,17 +295,27 @@ let init
 ;;
 
 
-let default_location() = !conf_default_location;;
+let lazy_init() =
+  if not !init_called then init()
+
+
+let default_location() = 
+  lazy_init();
+  !conf_default_location;;
 
 
 let meta_directory() =
+  lazy_init();
   if !conf_meta_directory = "none" then "" else !conf_meta_directory;;
 
 
-let search_path() = !conf_search_path;;
+let search_path() = 
+  lazy_init();
+  !conf_search_path;;
 
 
 let command which =
+  lazy_init();
   try 
     List.assoc which !conf_command
   with
@@ -268,34 +323,46 @@ let command which =
 ;;
 
 
-let ocaml_stdlib() = !conf_stdlib;;
+let ocaml_stdlib() = 
+  lazy_init();
+  !conf_stdlib;;
 
 
-let ocaml_ldconf() = !conf_ldconf;;
+let ocaml_ldconf() = 
+  lazy_init();
+  !conf_ldconf;;
 
+let ignore_dups_in() = 
+  lazy_init();
+  !conf_ignore_dups_in;;
 
 let package_directory pkg =
+  lazy_init();
   (Fl_package_base.query pkg).Fl_package_base.package_dir
 ;;
 
 
 let package_property predlist pkg propname =
+  lazy_init();
   let l = Fl_package_base.query pkg in
   Fl_metascanner.lookup propname predlist l.Fl_package_base.package_defs
 ;;
 
 
 let package_ancestors predlist pkg =
+  lazy_init();
   Fl_package_base.requires predlist pkg
 ;;
 
 
 let package_deep_ancestors predlist pkglist =
+  lazy_init();
   Fl_package_base.requires_deeply predlist pkglist
 ;;
 
 
 let resolve_path ?base p =
+  lazy_init();
   if p = "" then "" else (
     match p.[0] with
 	'^' | '+' ->
@@ -316,19 +383,21 @@ let resolve_path ?base p =
 		  let pkg = String.sub p 1 (String.length p - 1) in
 		  package_directory pkg
 	  )
-      | '/' ->
-	  p
       | _ ->
 	  ( match base with
 		None -> p
 	      | Some b ->
-		  Filename.concat b p
+		  if Filename.is_relative p then
+		    Filename.concat b p
+		  else
+		    p
 	  )
   )
 ;;
 
 
 let list_packages ?(tab = 20) ?(descr = false) ch =
+  lazy_init();
   let packages = Fl_package_base.list_packages() in
   let packages_sorted = List.sort compare packages in
 
@@ -356,6 +425,3 @@ let list_packages ?(tab = 20) ?(descr = false) ch =
     )
     packages_sorted
 ;;
-
-
-init();
